/****************************************************************************
*       cmdline.c
*      
* **********************************************************************
* *     Copyright (c) by Barak Pearlmutter and Kevin Lang, 1987-99.    *
* *     Copyright (c) by Alex Stuebinger, 1998-99.                     *
* *     Placed under the terms of the GNU General Public License       *
* *     as published by the Free Software Foundation.                  *
* **********************************************************************
*
*/

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include "config.h"
#include "data.h"
#include "cmdline.h"
#include "xmalloc.h"

enum {
  FLAG = 0,
  AUDIT_ARG,
  WORLD_ARG,
  CONTEXT_STACK_ARG,
  DUMP_ARG,
  DUMP_BASE_ARG,
  PREDUMP_GC,
  HEAP_ARG,
  HELP_ARG,
  NOCORE_ARG,
  TRACE_ARG,
  VALUE_STACK_ARG,
  QUIET_ARG,
  UNIMPLEMENTED_ARG
};


void 
usage_short (char *prog)
{
  fprintf (stdout,
	   "Usage: %s options...:\n"
	   /* "\t--audit [file]\n" */
	   "\t--world file\n"
	   "\t--context-stack blocks\n"
	   "\t--dump file\n"
	   "\t--dump-base b (0=ascii, 2=binary.  default=2)\n"
	   "\t--predump-gc bool\n"
	   "\t--heap blocks\n"
	   "\t--help\n"
	   "\t--large\n"
	   "\t--small\n"
	   /* "\t--trace what\n" */
	   "\t--value-stack blocks\n"
	   "\t--gc-noise\n"
	   "\t--version\n"
	   "\t--quiet\n",
	   prog
	   );
}


void
parse_cmd_line (int argc, char **argv)
{
  int retval, option_index = 0;

  /* check for correct endianity settings */
  /* main() routine sets byte_gender */
  if (BYTE_GENDER != byte_gender)
    {
      fprintf(stderr, "error: configured for incorrect endianity.\n");
      exit(EXIT_FAILURE);
    }

  /* parse command line arguments */
  while (1)
    {
      static struct option long_options[] = {
	{"audit", optional_argument, 0, AUDIT_ARG},
	{"world", required_argument, 0, WORLD_ARG},
	{"band", required_argument, 0, WORLD_ARG}, /* deprecated */
	{"context-stack", required_argument, 0, CONTEXT_STACK_ARG},
	{"dump", required_argument, 0, DUMP_ARG},
	{"dump-base", required_argument, 0, DUMP_BASE_ARG},
	{"predump-gc", required_argument, 0, PREDUMP_GC},
	{"heap", required_argument, 0, HEAP_ARG},
	{"help", no_argument, 0, HELP_ARG},
	{"nocore", no_argument, 0, NOCORE_ARG},
	{"trace", required_argument, 0, TRACE_ARG},
	{"value-stack", required_argument, 0, VALUE_STACK_ARG},
	{"gc-noise", 0, &gc_shutup, false},
	{"quiet", no_argument, 0, QUIET_ARG},
	{0, 0, 0, 0}};

      retval = getopt_long_only(argc,
				argv,
				"", /* oak99_short_options, */
				long_options,
				&option_index);

      if (retval == EOF)
	break;

      switch (retval)
	{
	default:
	  fprintf (stderr, "error: command line syntax\n");
	case '?':
	  /* getopt_long() already printed an error message. */
	  usage_short (argv[0]);
	  exit (EXIT_FAILURE);
	  break;

	unimplemented:
	case UNIMPLEMENTED_ARG:
	  fprintf (stderr, "error: unimplemented command line option \"%s\"",
		   long_options[option_index].name);
	  if (optarg)
	    fprintf (stderr, " with arg \"%s\"", optarg);
	  fprintf(stderr, "\n");
	  exit (EXIT_FAILURE);
	  break;

	case FLAG:
	  /* variable set by getopt() */
	  break;

	case AUDIT_ARG:
	  goto unimplemented;
	  break;

	case WORLD_ARG:
	  world_file_name = optarg;
	  break;

	case CONTEXT_STACK_ARG:
#ifdef STACKS_STATIC
	  fprintf (stderr, "Warning (command line parser): "
		   "stacks are static, cannot set context stack size.\n");
	  break;
#else
	  {
	    long blocks = atol (optarg);

	    context_stack_size = blocks * 1024 - 2;
	    context_stack_hysteresis =
	      blocks * 1024 / 64 < 32 ? 32 : blocks * 1024 / 64;
	    break;
	  }
#endif
	case DUMP_ARG:
	  dump_file_name = optarg; 
	  dump_flag = true;
	  break;

	case DUMP_BASE_ARG:
	  dump_flag = true;
	  dump_base = atoi (optarg);
	  if (dump_base != 2 && dump_base != 10 && dump_base != 16)
	    {
	      fprintf (stderr, "Error (command line parser): invalid"
		       " dump base %s.\n", optarg);
	      exit (EXIT_FAILURE);
	    }
	  break;

	case PREDUMP_GC:
	  gc_before_dump = atoi(optarg);
	  break;

	case HEAP_ARG:
	  {
	    long blocks = atol (optarg);
	    original_newspace_size = (size_t) blocks *1024;
	    break;
	  }
	case HELP_ARG:
	  usage_short (argv[0]);
	  exit (EXIT_SUCCESS);
	  break;

	case NOCORE_ARG:
	  goto unimplemented;
	  break;

	case TRACE_ARG:
	  goto unimplemented;
	  break;

	case VALUE_STACK_ARG:
#ifdef STACKS_STATIC
	  fprintf (stderr, "Warning (command line parser): "
		   "stacks are static, cannot set value stack size.\n");
	  break;
#else
	  {
	    long blocks = atol (optarg);

	    value_stack_size = blocks * 1024 - 2;
	    value_stack_hysteresis =
	      blocks * 1024 / 64 < 32 ? 32 : blocks * 1024 / 64;
	    break;
	  }
#endif

	case QUIET_ARG:
	  quiet_flag = true;
	  break;

	}
    }

  /* handle remainder of command line */
  /* put in variables accessed by Oaklisp-level argline instructions */

  if (optind < argc)
    {
      fprintf (stderr, "error: superfluous material on command line.\n");
      usage_short(argv[0]);
      exit (EXIT_FAILURE);
    }

  /* other setup */

#ifdef NOTBAP
#ifndef STACKS_STATIC
  value_stack_size = config.value_stack_size;
  value_stack_hysteresis = config.value_stack_hysteresis;
  context_stack_size = config.context_stack_size;
  context_stack_hysteresis = config.context_stack_hysteresis;
#endif
  original_newspace_size = config.heap_size;
#endif

  original_newspace_size = 1024*1024;
  value_stack_size = 1024*4;
  value_stack_hysteresis = 10;
  context_stack_size = 1024*4;
  context_stack_hysteresis = 10;

  return;
}
